@@ -30,6 +30,8 @@
 #include "gitg-commit.h"
 #include "gitg-utils.h"
 #include "gitg-diff-view.h"
+#include "gitg-preferences.h"
+#include "gitg-data-binding.h"
 
 #define GITG_COMMIT_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GITG_TYPE_COMMIT_VIEW, GitgCommitViewPrivate))
 #define CATEGORY_UNSTAGE_HUNK "CategoryUnstageHunk"
@@ -70,6 +72,7 @@ struct _GitgCommitViewPrivate
 	GtkSourceView *changes_view;
 	GtkTextView *comment_view;
 	GtkCheckButton *check_button_signed_off_by;
+	GtkCheckButton *check_button_amend;
 	
 	GtkHScale *hscale_context;
 	gint context_size;
@@ -91,7 +94,7 @@ struct _GitgCommitViewPrivate
 
 static void gitg_commit_view_buildable_iface_init(GtkBuildableIface *iface);
 
-G_DEFINE_TYPE_EXTENDED(GitgCommitView, gitg_commit_view, GTK_TYPE_HPANED, 0,
+G_DEFINE_TYPE_EXTENDED(GitgCommitView, gitg_commit_view, GTK_TYPE_VPANED, 0,
 	G_IMPLEMENT_INTERFACE(GTK_TYPE_BUILDABLE, gitg_commit_view_buildable_iface_init));
 
 static GtkBuildableIface parent_iface;
@@ -121,6 +124,7 @@ static void on_revert_changes(GtkAction *action, GitgCommitView *view);
 static void on_ignore_file(GtkAction *action, GitgCommitView *view);
 static void on_unstage_changes(GtkAction *action, GitgCommitView *view);
 
+static void on_check_button_amend_toggled (GtkToggleButton *button, GitgCommitView *view);
 
 static void
 gitg_commit_view_finalize(GObject *object)
@@ -227,8 +231,6 @@ on_changes_update(GitgRunner *runner, gchar **buffer, GitgCommitView *view)
 		gtk_text_buffer_insert(buf, &iter, "\n", -1);
 	}
 	
-	gitg_utils_guess_content_type(GTK_TEXT_BUFFER(buf));
-	
 	if (gtk_source_buffer_get_language(GTK_SOURCE_BUFFER(buf)) == NULL)
 	{
 		gchar *content_type = gitg_utils_guess_content_type(GTK_TEXT_BUFFER(buf));
@@ -240,7 +242,7 @@ on_changes_update(GitgRunner *runner, gchar **buffer, GitgCommitView *view)
 		}
 		else if (content_type)
 		{
-			GtkSourceLanguage *language = gitg_utils_get_language(content_type);
+			GtkSourceLanguage *language = gitg_utils_get_language(NULL, content_type);
 			set_language(view, language);
 			gtk_widget_set_sensitive(GTK_WIDGET(view->priv->hscale_context), FALSE);
 		}
@@ -269,7 +271,84 @@ set_current_file(GitgCommitView *view, GitgChangedFile *file, GitgChangedFileCha
 }
 
 static gboolean
-check_selection(GtkTreeSelection *selection, GtkTreeModel **model, GtkTreeIter *iter, GitgCommitView *view)
+get_selected_files(GtkTreeView             *tree_view,
+                   GList                  **files,
+                   GList                  **paths,
+                   GitgChangedFileChanges  *changes,
+                   GitgChangedFileStatus   *status)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
+
+	if (files == NULL && changes == NULL && status == NULL && paths == NULL)
+		return gtk_tree_selection_count_selected_rows(selection) != 0;
+	
+	GtkTreeModel *model;
+	GList *items = gtk_tree_selection_get_selected_rows(selection, &model);
+	
+	if (files)
+		*files = NULL;
+	
+	if (paths)
+		*paths = NULL;
+	
+	if (!items)
+		return FALSE;
+	
+	if (changes)
+		*changes = ~0;
+	
+	if (status)
+		*status = -1;
+	
+	GList *item;
+	GitgChangedFile *file;
+
+	for (item = items; item; item = g_list_next(item))
+	{
+		GtkTreeIter iter;
+
+		gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)item->data);
+		gtk_tree_model_get(model, &iter, COLUMN_FILE, &file, -1);
+		
+		if (changes)
+			*changes &= gitg_changed_file_get_status(file);
+		
+		GitgChangedFileStatus s = gitg_changed_file_get_status(file);
+		
+		if (status)
+		{
+			if (*status != -1 && *status != s)
+				*status = GITG_CHANGED_FILE_STATUS_NONE;
+			else
+				*status = s;
+		}
+		
+		if (files)
+			*files = g_list_prepend(*files, file);
+		else
+			g_object_unref(file);
+	}
+	
+	if (paths)
+	{
+		*paths = items;
+	}
+	else
+	{
+		g_list_foreach(items, (GFunc)gtk_tree_path_free, NULL);
+		g_list_free(items);
+	}
+	
+	if (files)
+		*files = g_list_reverse(*files);
+	
+	return TRUE;
+}
+
+static gboolean
+check_selection(GtkTreeView    *tree_view, 
+                GtkTreeIter    *iter, 
+                GitgCommitView *view)
 {
 	if (view->priv->update_id)
 		g_signal_handler_disconnect(view->priv->runner, view->priv->update_id);
@@ -287,14 +366,28 @@ check_selection(GtkTreeSelection *selection, GtkTreeModel **model, GtkTreeIter *
 	gtk_source_buffer_remove_source_marks(GTK_SOURCE_BUFFER(buffer), &start, &end, CATEGORY_STAGE_HUNK);
 	gtk_text_buffer_set_text(gtk_text_view_get_buffer(tv), "", -1);
 	
-	if (!gtk_tree_selection_get_selected(selection, model, iter))
+	GList *paths;
+	gboolean ret;
+	get_selected_files(tree_view, NULL, &paths, NULL, NULL);
+	
+	if (g_list_length(paths) != 1)
 	{
 		set_current_file(view, NULL, GITG_CHANGED_FILE_CHANGES_NONE);
 		gtk_widget_set_sensitive(GTK_WIDGET(view->priv->hscale_context), FALSE);
-		return FALSE;
+		ret = FALSE;
+	}
+	else
+	{
+		if (iter)
+			gtk_tree_model_get_iter(gtk_tree_view_get_model(tree_view),
+			                        iter,
+			                        (GtkTreePath *)paths->data);
+		ret = TRUE;
 	}
 	
-	return TRUE;
+	g_list_foreach(paths, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free(paths);
+	return ret;
 }
 
 static void
@@ -309,9 +402,10 @@ unstaged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	
-	if (!check_selection(selection, &model, &iter, view))
+	if (!check_selection(view->priv->tree_view_unstaged, &iter, view))
 		return;
 	
+	model = gtk_tree_view_get_model(view->priv->tree_view_unstaged);
 	unselect_tree_view(view->priv->tree_view_staged);
 	
 	GitgChangedFile *file;
@@ -338,7 +432,10 @@ unstaged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 			}
 			else
 			{
-				GtkSourceLanguage *language = gitg_utils_get_language(content_type);
+				gchar *basename = g_file_get_basename(f);
+				GtkSourceLanguage *language = gitg_utils_get_language(basename, content_type);
+				g_free(basename);
+
 				set_language(view, language);
 				gtk_widget_set_sensitive(GTK_WIDGET(view->priv->hscale_context), FALSE);
 				
@@ -349,6 +446,8 @@ unstaged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 				g_object_unref(stream);
 			}
 		}
+
+		g_free(content_type);
 	}
 	else
 	{
@@ -377,10 +476,12 @@ staged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	
-	if (!check_selection(selection, &model, &iter, view))
+	if (!check_selection(view->priv->tree_view_staged, &iter, view))
 		return;
 	
+	model = gtk_tree_view_get_model(view->priv->tree_view_staged);
 	unselect_tree_view(view->priv->tree_view_unstaged);
+
 	GitgChangedFile *file;
 	
 	gtk_tree_model_get(model, &iter, COLUMN_FILE, &file, -1);
@@ -388,7 +489,6 @@ staged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 	
 	GFile *f = gitg_changed_file_get_file(file);
 	gchar *path = gitg_repository_relative(view->priv->repository, f);
-	g_object_unref(f);
 	
 	if (status == GITG_CHANGED_FILE_STATUS_NEW)
 	{
@@ -402,13 +502,16 @@ staged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 		}
 		else
 		{
-			set_language(view, gitg_utils_get_language(content_type));
+			gchar *basename = g_file_get_basename(f);
+			GtkSourceLanguage *language = gitg_utils_get_language(basename, content_type);
+			g_free(basename);
+
 			gtk_widget_set_sensitive(GTK_WIDGET(view->priv->hscale_context), FALSE);
 
 			connect_update(view);
 
 			gchar *indexpath = g_strconcat(":0:", path, NULL);
-			gitg_repository_run_commandv(view->priv->repository, view->priv->runner, NULL, "show", indexpath, NULL);
+			gitg_repository_run_commandv(view->priv->repository, view->priv->runner, NULL, "show", "--encoding=UTF-8", indexpath, NULL);
 			g_free(indexpath);
 		}
 		
@@ -428,6 +531,7 @@ staged_selection_changed(GtkTreeSelection *selection, GitgCommitView *view)
 		g_free(head);
 	}
 
+	g_object_unref(f);
 	g_free(path);	
 
 	set_current_file(view, file, GITG_CHANGED_FILE_CHANGES_CACHED);	
@@ -671,6 +775,142 @@ initialize_buffer(GitgCommitView *view)
 	return buffer;
 }
 
+static GtkTargetEntry dnd_entries[] = {
+	{"text/uri-list", 0, 1}
+};
+
+static void
+on_tree_view_drag_data_get (GtkWidget        *widget,
+                            GdkDragContext   *context,
+                            GtkSelectionData *selection,
+                            guint             info,
+                            guint             time,
+                            GitgCommitView   *view)
+{
+	GList *selected;
+	GList *item;
+	gchar **uris;
+	guint i = 0;
+
+	get_selected_files(GTK_TREE_VIEW(widget), &selected, NULL, NULL, NULL);	
+	uris = g_new(gchar *, g_list_length(selected) + 1);
+	
+	for (item = selected; item; item = g_list_next(item))
+	{
+		GitgChangedFile *file = GITG_CHANGED_FILE (item->data);
+		GFile *gf = gitg_changed_file_get_file(file);
+	
+		uris[i++] = g_file_get_uri(gf);
+		g_object_unref(gf);
+	}
+	
+	uris[i] = NULL;
+	gtk_selection_data_set_uris(selection, uris);
+
+	g_strfreev(uris);
+
+	g_list_foreach(selected, (GFunc)g_object_unref, NULL);
+	g_list_free(selected);
+}
+
+static void
+on_tree_view_staged_drag_data_received(GtkWidget        *widget,
+                                       GdkDragContext   *drag_context,
+                                       gint              x,
+                                       gint              y,
+                                       GtkSelectionData *data,
+                                       guint             info,
+                                       guint             time,
+                                       GitgCommitView   *view)
+{
+	/* Stage all the files dropped on this */
+	gchar **uris = gtk_selection_data_get_uris(data);
+	gchar **uri;
+
+	for (uri = uris; *uri; ++uri)
+	{
+		GFile *file = g_file_new_for_uri(*uri);
+		GitgChangedFile *f;
+
+		f = gitg_commit_find_changed_file(view->priv->commit, file);
+		
+		if (f && (gitg_changed_file_get_changes(f) & GITG_CHANGED_FILE_CHANGES_UNSTAGED))
+		{
+			gitg_commit_stage(view->priv->commit, f, NULL, NULL);
+		}
+		
+		g_object_unref(f);
+		g_object_unref(file);
+	}
+}
+
+static void
+on_tree_view_unstaged_drag_data_received(GtkWidget        *widget,
+                                         GdkDragContext   *drag_context,
+                                         gint              x,
+                                         gint              y,
+                                         GtkSelectionData *data,
+                                         guint             info,
+                                         guint             time,
+                                         GitgCommitView   *view)
+{
+	/* Unstage all the files dropped on this */
+	gchar **uris = gtk_selection_data_get_uris(data);
+	gchar **uri;
+	
+	for (uri = uris; *uri; ++uri)
+	{
+		GFile *file = g_file_new_for_uri(*uri);
+		GitgChangedFile *f;
+
+		f = gitg_commit_find_changed_file(view->priv->commit, file);
+		
+		if (f && (gitg_changed_file_get_changes(f) & GITG_CHANGED_FILE_CHANGES_CACHED))
+		{
+			gitg_commit_unstage(view->priv->commit, f, NULL, NULL);
+		}
+		
+		g_object_unref(f);
+		g_object_unref(file);
+	}
+}
+
+static void
+initialize_dnd(GitgCommitView *view, 
+               GtkTreeView    *tree_view,
+               GCallback       drag_data_received)
+{
+	gtk_tree_view_enable_model_drag_dest(tree_view,
+	                                     dnd_entries,
+	                                     G_N_ELEMENTS(dnd_entries),
+	                                     GDK_ACTION_COPY);
+
+	gtk_tree_view_enable_model_drag_source(tree_view,
+	                                      GDK_BUTTON1_MASK,
+	                                      dnd_entries,
+	                                      G_N_ELEMENTS(dnd_entries),
+	                                      GDK_ACTION_COPY);
+
+	g_signal_connect(tree_view, "drag-data-get", G_CALLBACK(on_tree_view_drag_data_get), view);
+	g_signal_connect(tree_view, "drag-data-received", G_CALLBACK(drag_data_received), view);
+}
+
+static void
+initialize_dnd_staged(GitgCommitView *view)
+{
+	initialize_dnd(view,
+	               view->priv->tree_view_staged,
+	               G_CALLBACK(on_tree_view_staged_drag_data_received));
+}
+
+static void
+initialize_dnd_unstaged(GitgCommitView *view)
+{
+	initialize_dnd(view,
+	               view->priv->tree_view_unstaged,
+	               G_CALLBACK(on_tree_view_unstaged_drag_data_received));
+}
+
 static void
 gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 {
@@ -680,7 +920,18 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	/* Store widgets */
 	GitgCommitView *self = GITG_COMMIT_VIEW(buildable);
 	
-	self->priv->ui_manager = g_object_ref(gtk_builder_get_object(builder, "uiman"));
+	GtkBuilder *b = gitg_utils_new_builder("gitg-commit-menu.ui");
+	self->priv->ui_manager = g_object_ref(gtk_builder_get_object(b, "uiman"));
+
+	g_signal_connect(gtk_builder_get_object(b, "StageChangesAction"), "activate", G_CALLBACK(on_stage_changes), self);
+	g_signal_connect(gtk_builder_get_object(b, "RevertChangesAction"), "activate", G_CALLBACK(on_revert_changes), self);
+	g_signal_connect(gtk_builder_get_object(b, "IgnoreFileAction"), "activate", G_CALLBACK(on_ignore_file), self);
+	g_signal_connect(gtk_builder_get_object(b, "UnstageChangesAction"), "activate", G_CALLBACK(on_unstage_changes), self);
+
+	self->priv->group_context = GTK_ACTION_GROUP(gtk_builder_get_object(b, "action_group_commit_context"));
+	
+	g_object_unref(b);
+	
 	self->priv->tree_view_unstaged = GTK_TREE_VIEW(gtk_builder_get_object(builder, "tree_view_unstaged"));
 	self->priv->tree_view_staged = GTK_TREE_VIEW(gtk_builder_get_object(builder, "tree_view_staged"));
 	
@@ -693,9 +944,21 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	self->priv->changes_view = GTK_SOURCE_VIEW(gtk_builder_get_object(builder, "source_view_changes"));
 	self->priv->comment_view = GTK_TEXT_VIEW(gtk_builder_get_object(builder, "text_view_comment"));
 	self->priv->check_button_signed_off_by = GTK_CHECK_BUTTON(gtk_builder_get_object(builder, "check_button_signed_off_by"));
+	self->priv->check_button_amend = GTK_CHECK_BUTTON(gtk_builder_get_object(builder, "check_button_amend"));
+	
+	GitgPreferences *preferences = gitg_preferences_get_default();
+	
+	gitg_data_binding_new(preferences, "message-show-right-margin",
+	                      self->priv->comment_view, "show-right-margin");
+
+	gitg_data_binding_new(preferences, "message-right-margin-at",
+	                      self->priv->comment_view, "right-margin-position");
 	
 	self->priv->hscale_context = GTK_HSCALE(gtk_builder_get_object(builder, "hscale_context"));
-	self->priv->group_context = GTK_ACTION_GROUP(gtk_builder_get_object(builder, "action_group_commit_context"));
+	gtk_range_set_value (GTK_RANGE (self->priv->hscale_context), 3);
+	
+	initialize_dnd_staged(self);
+	initialize_dnd_unstaged(self);
 	
 	GtkIconTheme *theme = gtk_icon_theme_get_default();
 	GdkPixbuf *pixbuf = gtk_icon_theme_load_icon(theme, GTK_STOCK_ADD, 12, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
@@ -726,8 +989,16 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	set_icon_data_func(self, self->priv->tree_view_unstaged, GTK_CELL_RENDERER(gtk_builder_get_object(builder, "unstaged_cell_renderer_icon")));
 	set_icon_data_func(self, self->priv->tree_view_staged, GTK_CELL_RENDERER(gtk_builder_get_object(builder, "staged_cell_renderer_icon")));
 	
-	g_signal_connect(gtk_tree_view_get_selection(self->priv->tree_view_unstaged), "changed", G_CALLBACK(unstaged_selection_changed), self);
-	g_signal_connect(gtk_tree_view_get_selection(self->priv->tree_view_staged), "changed", G_CALLBACK(staged_selection_changed), self);
+	GtkTreeSelection *selection;
+	
+	selection = gtk_tree_view_get_selection(self->priv->tree_view_unstaged);
+	
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+	g_signal_connect(selection, "changed", G_CALLBACK(unstaged_selection_changed), self);
+	
+	selection = gtk_tree_view_get_selection(self->priv->tree_view_staged);
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+	g_signal_connect(selection, "changed", G_CALLBACK(staged_selection_changed), self);
 	
 	g_signal_connect(self->priv->tree_view_unstaged, "event-after", G_CALLBACK(on_unstaged_button_press), self);
 	g_signal_connect(self->priv->tree_view_staged, "event-after", G_CALLBACK(on_staged_button_press), self);
@@ -743,10 +1014,10 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	
 	g_signal_connect(self->priv->hscale_context, "value-changed", G_CALLBACK(on_context_value_changed), self);
 	
-	g_signal_connect(gtk_builder_get_object(builder, "StageChangesAction"), "activate", G_CALLBACK(on_stage_changes), self);
-	g_signal_connect(gtk_builder_get_object(builder, "RevertChangesAction"), "activate", G_CALLBACK(on_revert_changes), self);
-	g_signal_connect(gtk_builder_get_object(builder, "IgnoreFileAction"), "activate", G_CALLBACK(on_ignore_file), self);
-	g_signal_connect(gtk_builder_get_object(builder, "UnstageChangesAction"), "activate", G_CALLBACK(on_unstage_changes), self);
+	g_signal_connect (self->priv->check_button_amend,
+	                  "toggled",
+	                  G_CALLBACK (on_check_button_amend_toggled),
+	                  self);
 }
 
 static void
@@ -1176,10 +1447,11 @@ on_commit_clicked(GtkButton *button, GitgCommitView *view)
 	}
 	
 	gboolean signoff = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view->priv->check_button_signed_off_by));
-	
+	gboolean amend = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->priv->check_button_amend));
+
 	GError *error = NULL;
 	
-	if (!gitg_commit_commit(view->priv->commit, comment, signoff, &error))
+	if (!gitg_commit_commit(view->priv->commit, comment, signoff, amend, &error))
 	{
 		if (error && error->domain == GITG_COMMIT_ERROR && error->code == GITG_COMMIT_ERROR_SIGNOFF)
 			show_error(view, _("Your user name or email could not be retrieved for use in the sign off message"));
@@ -1192,6 +1464,8 @@ on_commit_clicked(GtkButton *button, GitgCommitView *view)
 	else
 	{
 		gtk_text_buffer_set_text(gtk_text_view_get_buffer(view->priv->comment_view), "", -1);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->check_button_amend), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->check_button_signed_off_by), FALSE);
 	}
 	
 	g_free(comment);
@@ -1211,36 +1485,26 @@ on_context_value_changed(GtkHScale *scale, GitgCommitView *view)
 static gboolean
 set_unstaged_popup_status(GitgCommitView *view)
 {
-	GtkTreeSelection *selection = gtk_tree_view_get_selection(view->priv->tree_view_unstaged);
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	
-	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+	GitgChangedFileChanges changes;
+	GitgChangedFileStatus status;
+
+	if (!get_selected_files(view->priv->tree_view_unstaged, NULL, NULL, &changes, &status))
 		return FALSE;
 	
-	GitgChangedFile *file;
-	gtk_tree_model_get(model, &iter, COLUMN_FILE, &file, -1);
-	
 	GtkAction *revert = gtk_ui_manager_get_action(view->priv->ui_manager, "/ui/popup_commit_stage/RevertChangesAction");
 	GtkAction *ignore = gtk_ui_manager_get_action(view->priv->ui_manager, "/ui/popup_commit_stage/IgnoreFileAction");
-	gboolean isnew = gitg_changed_file_get_status(file) == GITG_CHANGED_FILE_STATUS_NEW;
+	gboolean isnew = status == GITG_CHANGED_FILE_STATUS_NEW;
 
 	gtk_action_set_visible(revert, !isnew);
 	gtk_action_set_visible(ignore, isnew);
 	
-	g_object_unref(file);
-	
 	return TRUE;
 }
 
 static gboolean
 set_staged_popup_status(GitgCommitView *view)
 {
-	GtkTreeSelection *selection = gtk_tree_view_get_selection(view->priv->tree_view_staged);
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	
-	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+	if (!get_selected_files(view->priv->tree_view_staged, NULL, NULL, NULL, NULL))
 		return FALSE;
 	
 	return TRUE;	
@@ -1320,7 +1584,6 @@ on_stage_changes(GtkAction *action, GitgCommitView *view)
 			return;
 		
 		gitg_commit_stage(view->priv->commit, file, NULL, NULL);
-		g_object_unref(file);
 	}
 	else
 	{
@@ -1435,3 +1698,27 @@ on_changes_view_popup_menu(GtkTextView *textview, GtkMenu *menu, GitgCommitView
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), revert);
 	}
 }
+
+static void
+on_check_button_amend_toggled (GtkToggleButton *button, GitgCommitView *view)
+{
+	gboolean active = gtk_toggle_button_get_active (button);
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view->priv->comment_view);
+	GtkTextIter start;
+	GtkTextIter end;
+	
+	gtk_text_buffer_get_bounds (buffer, &start, &end);
+	
+	if (active && gtk_text_iter_compare (&start, &end) == 0)
+	{
+		// Get last commit message
+		gchar *message = gitg_commit_amend_message (view->priv->commit);
+		
+		if (message)
+		{
+			gtk_text_buffer_set_text (buffer, message, -1);
+		}
+		
+		g_free (message);
+	}
+}
